/***
 * 
 * Felipe Brahm
 * Ayudante de Ctedra
 * Introduccin a la Programacin - IIC1102 Seccin 4
 * Pontificia Universidad Catlica de Chile
 *
 * lebrahm <at> uc <punto> cl
 * ICQ#50566818
 *
 * (CC)2006
 * 
 * Esta solucin fue tomada de http://www.csaszar.org
 * y luego modificada para el lenguaje Java.
 * 
 * */

import iic1102Package.*;

public class Laberinto {
	
	/* Constantes que definen la dimensin del laberinto */
	private static int FILAS = 13;
	private static int COLUMNAS = 21;

	/* Caracteres que se almacenan en las posiciones del laberinto:    */
	/*    X se usa para las paredes   */
	/*    ' ' es un espacio en blanco, para los caminos libres         */
	/*    '*' es la marca que seala las posiciones visitadas (camino) */
	private static char L = 'X';
	private static char E = ' ';
	private static char MARCA = '*';

	/* Matriz global que representa el laberinto. En la inicializacin, */
	/* L representa un ladrillo en la pared y E un espacio en blanco    */
	/* Obviamente esta parte se podra cambiar para que el laberinto se	*/
	/* genere automticamente o algo as...								*/
	private static char[][] lab =
	   { { L, L, L, L, L, L, L, L, L, L, L, L, L, L, L, L, L, L, L, L, L },
	     { E, E, E, L, E, L, E, L, E, L, E, E, E, L, E, E, E, E, E, E, L },
	     { L, L, E, L, E, E, E, L, E, E, E, L, E, L, E, L, E, E, L, E, L },
	     { L, E, E, E, E, L, E, L, L, L, E, L, E, L, E, L, L, E, L, E, L },
	     { L, E, L, L, L, L, E, L, E, L, E, L, E, L, L, L, E, E, L, E, L },
	     { L, E, E, E, L, E, E, L, E, L, E, L, E, E, E, E, E, L, L, E, L },
	     { L, E, L, L, L, L, L, L, E, L, E, L, L, E, L, E, E, E, E, E, L },
	     { L, E, E, E, E, E, E, E, E, E, E, E, L, E, L, E, L, L, E, L, L },
	     { L, L, L, L, L, L, L, L, L, L, L, E, L, E, L, E, E, L, E, L, L },
	     { L, E, E, E, E, E, E, E, L, E, E, E, L, L, L, L, L, L, E, L, L },
	     { L, E, L, L, L, L, L, E, E, E, L, L, L, E, L, E, E, E, E, E, L },
	     { L, E, E, L, E, E, E, E, L, E, E, L, E, E, E, E, L, L, L, E, E },
	     { L, L, L, L, L, L, L, L, L, L, L, L, L, L, L, L, L, L, L, L, L }
	   };
	
	private static int x0 = 1;
	private static int y0 = 0;
	private static int xf = 11;
	private static int yf = 20;

	public static void main(String[] args) {
		
		boolean ok; //para saber si se obtuvo solucin
		Interfaz.MostrarMensajeConsola("Laberinto:");
		desplegarLaberinto(lab);//despliega el laberinto sin resolver
		/* Resuelve el laberinto desde la entrada: (x0,y0) */
		ok = recorre(x0, y0);
		if(!ok) //si no se pudo resolver se muestra un mensaje
			Interfaz.MostrarMensajeConsola("\nLaberinto sin solucin.");
		else {
			Interfaz.MostrarMensajeConsola("\nLaberinto con Solucin:");
			desplegarLaberinto(lab); //despliega el laberinto resuelto (con el camino)
		}
		
	}
	
	public static boolean recorre(int fil, int col) {

		boolean listo = false; // Indica si se ha encontrado la salida
		/* Se marca la casilla como visitada */
		lab[fil][col] = MARCA;
		/* Se controla la condicion de trmino de recursividad: */
		/*    " Llegamos a la salida? "                         */
		if (fil == xf && col == yf)
			return(true);
		if (!listo && valida(fil,col-1))   /* Intento a la izquierda */
			listo = recorre(fil,col-1);
		if (!listo && valida(fil,col+1))   /* Intento a la derecha */
			listo = recorre(fil,col+1);
		if (!listo && valida(fil-1,col))   /* Intento hacia arriba */
			listo = recorre(fil-1,col);
		if (!listo && valida(fil+1,col))   /* Intento hacia abajo */
			listo = recorre(fil+1,col);
		/* Si no se logr resolver el laberinto desde esta posicion, se   */
		/* desmarca la casilla pues no ser parte de la solucion. En este */
		/* caso se retornar falso lo que provocar que se retroceda.     */
		if (!listo)
			lab[fil][col] = E;
		/* Se retorna true/false dependiendo de si se encontr solucin */
		return(listo);
	}
	
	public static void desplegarLaberinto(char[][] lab) {
		
		Interfaz.MostrarMensajeConsola("");
		
		for(int i = 0; i < FILAS; ++i) {
			
			String texto = "";
			
			for(int j = 0; j < COLUMNAS; ++j) {
				
				texto += lab[i][j] + " ";
			}

			Interfaz.MostrarMensajeConsola(texto);
			
		}
		
	}
	
	public static boolean valida(int f, int c) {
		
		//controla si la posicin est fuera del laberinto
		if (f < 0 || f == FILAS || c < 0 || c == COLUMNAS)
			return false;

		//controla si la posicin ya fue visitada o es muro
		if (lab[f][c] == MARCA || lab[f][c] == L)
			return false;

		return true;

	}

}